@@ -3,26 +3,43 @@
 import slumber
 
 
+SYSTEM_DATABASE = '_system'
+
 class Client(object):
 
     class_instance = None
 
 
-    def __init__(self, hostname, protocol='http', port=8529, database='_system'):
+    def __init__(self, hostname, protocol='http', port=8529, database=SYSTEM_DATABASE):
         """
         """
 
         Client.class_instance = self
 
-        url = '%s://%s:%s/_db/%s/_api/' % (protocol, hostname, port, database)
+        self.hostname = hostname
+        self.protocol = protocol
+        self.port = port
+        self.database = database
+
+        self._create_api()
+
+    def set_database(self, name):
+        """
+        """
+
+        self.database = name
+        self._create_api()
+
+    def _create_api(self):
+
+        url = '%s://%s:%s/_db/%s/_api/' % (self.protocol, self.hostname, self.port, self.database)
         self.api = slumber.API(url, append_slash=False)
 
-
     @classmethod
-    def instance(cls, hostname=None, protocol=None, port=None):
+    def instance(cls, hostname=None, protocol=None, port=None, database=None):
 
         if cls.class_instance is None:
-            cls.class_instance = Client(hostname=hostname, protocol=protocol, port=port)
+            cls.class_instance = Client(hostname=hostname, protocol=protocol, port=port, database=database)
         else:
             if hostname is not None:
                 cls.class_instance.hostname = hostname
@@ -33,6 +50,11 @@ class Client(object):
             if port is not None:
                 cls.class_instance.hostname = port
 
+            if database is not None:
+                cls.class_instance.database = database
+
+            cls.class_instance._create_api()
+
         return cls.class_instance
 
     def collection(self, name):
@@ -246,7 +268,7 @@ class Query(object):
                 for result_key in result_dict:
                     result_value = result_dict[result_key]
 
-                    doc.setData(key=result_key, value=result_value)
+                    doc.set(key=result_key, value=result_value)
 
                 result.append(doc)
 
@@ -326,6 +348,47 @@ class Database(object):
 
         return db
 
+    @classmethod
+    def get_all(cls):
+        """
+        """
+
+        api = Client.instance().api
+
+        data = api.database.get()
+
+        database_names = data['result']
+        databases = []
+
+        for name in database_names:
+            db = Database(name=name, api=api)
+            databases.append(db)
+
+        return databases
+
+
+    @classmethod
+    def remove(cls, name):
+        """
+        """
+
+        client = Client.instance()
+
+        new_current_database = None
+
+        if client.database != name:
+            new_current_database = name
+
+        # Deletions are only possible from the system database
+        client.set_database(name=SYSTEM_DATABASE)
+
+        api = client.api
+        api.database(name).delete()
+
+        if new_current_database:
+            client.set_database(name=new_current_database)
+
+
     def __init__(self, name, api, **kwargs):
         """
         """
@@ -333,11 +396,11 @@ class Database(object):
         self.name = name
         self.api = api
 
-    def create_collection(self, name):
+    def create_collection(self, name, type=2):
         """
         """
 
-        return Collection.create(name=name, database=self)
+        return Collection.create(name=name, type=type)
 
 
 class Collection(object):
@@ -365,17 +428,45 @@ class Collection(object):
 
         return collection
 
+    @classmethod
+    def get_loaded_collection(cls, name):
+        """
+        """
+
+        api = Client.instance().api
+
+        data = api.collection(name).properties.get()
+
+        collection = Collection(
+            name=name,
+            api_resource=api.collection,
+            api=api,
+            kwargs=data
+        )
+
+        return collection
+
+    @classmethod
+    def remove(cls, name):
+        """
+        """
+
+        api = Client.instance().api
+
+        api.collection(name).delete()
+
+
     def __init__(self, name, api, **kwargs):
         """
         """
         self.name = name
 
-        self.setData(**kwargs)
+        self.set_data(**kwargs)
 
         self.resource = api.collection
         self.api = api
 
-    def setData(self, **kwargs):
+    def set_data(self, **kwargs):
         """
         """
 
@@ -394,6 +485,26 @@ class Collection(object):
         else:
             self.isVolatile = False
 
+        if 'doCompact' in kwargs:
+            self.doCompact = kwargs['doCompact']
+        else:
+            self.doCompact = None
+
+        if 'journalSize' in kwargs:
+            self.journalSize = kwargs['journalSize']
+        else:
+            self.journalSize = -1
+
+        if 'numberOfShards' in kwargs:
+            self.numberOfShards = kwargs['numberOfShards']
+        else:
+            self.numberOfShards = 0
+
+        if 'shardKeys' in kwargs:
+            self.shardKeys = kwargs['shardKeys']
+        else:
+            self.shardKeys = None
+
         if 'isSystem' in kwargs:
             self.isSystem = kwargs['isSystem']
         else:
@@ -415,10 +526,17 @@ class Collection(object):
 
         data = self.resource.get()
 
-        self.setData(**data)
+        self.set_data(**data)
 
         return data
 
+    def get_figures(self):
+        """
+        """
+
+        data = self.resource(self.name).figures.get()
+        return data['figures']
+
     def create_document(self):
         """
         """
@@ -518,7 +636,7 @@ class Document(object):
 
         self.is_loaded = False
 
-    def get(self):
+    def retrieve(self):
         """
         """
 
@@ -533,18 +651,18 @@ class Document(object):
 
         self.resource(self.id).patch(data=self.data)
 
-    def getData(self, key):
+    def get(self, key):
         """
         """
 
         if not self.is_loaded:
-            self.get()
+            self.retrieve()
 
             self.is_loaded = True
 
         return self.data[key]
 
-    def setData(self, key, value):
+    def set(self, key, value):
         """
         """
 
